﻿package com.patrickmatte.physicsLiquid {
	
	/**
	* @author Patrick Matte
	*/
	
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.filters.BlurFilter;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	
	import org.cove.ape.APEngine;
	import org.cove.ape.AbstractParticle;
	import org.cove.ape.CircleParticle;
	import org.cove.ape.Group;
	import org.cove.ape.RectangleParticle;
	import org.cove.ape.Vector;

	public dynamic class PhysicsLiquid extends MovieClip {
		
		private var bottle:Bottle;
		private var fluids:Array = new Array();
		private var fluidParticles:Array = new Array();
		private var floorGroup:Group;
		
		public var floor:MovieClip;
		private var floorParticle:AbstractParticle;
		
		public var bottom:MovieClip;
		private var bottomParticle:RectangleParticle;
		
		public var physicsContainer:MovieClip;
		
		private var bitmap:Bitmap = new Bitmap();
		private var bitmapData:BitmapData;
		private var bitmapDataCopy:BitmapData;
		
		private var blurFilter:BlurFilter = new BlurFilter(15, 15, 3);
		private var point:Point = new Point(0, 0);
		
		private var offsetY:Number;
		public var fullBottle:MovieClip;
		
		private var min:Number = 280;
		private var max:Number = 600;
		
		private var floorTargetY:Number;
		
		public function PhysicsLiquid() {
			super();
			bitmapData = new BitmapData(250, 450, true, 0xFF000000);
			bitmapDataCopy = new BitmapData(250, 450, false, 0xFF000000);
			
			var testBitmap:Bitmap = new Bitmap(bitmapDataCopy, "auto", true);
			testBitmap.x = 515;
			testBitmap.y = 100;
			testBitmap.scaleX = testBitmap.scaleY = 0.65;
			addChild(testBitmap);
			
			var testBitmap2:Bitmap = new Bitmap(bitmapData, "auto", true);
			testBitmap2.x = 700;
			testBitmap2.y = 100;
			testBitmap2.scaleX = testBitmap2.scaleY = 0.65;
			addChild(testBitmap2);
			
			bitmap = new Bitmap(bitmapData);
			addChildAt(bitmap, 0);
			
			bitmap.cacheAsBitmap = true;
			fullBottle.cacheAsBitmap = true;
			fullBottle.mask = bitmap;
			
			physicsContainer.x = 285;
			physicsContainer.y = 100;
			physicsContainer.scaleX = physicsContainer.scaleY = 0.65;
			
			floorTargetY = floor.y;
			floor.buttonMode = true;
			floor.addEventListener(MouseEvent.MOUSE_DOWN, floorMouseDownHandler);
			
			APEngine.init(1/2);
			APEngine.container = physicsContainer;
			APEngine.addMasslessForce(new Vector(0, 5));
			
			bottle = new Bottle(this);
			APEngine.addGroup(bottle);
			
			floorParticle = new RectangleParticle(floor.x, floor.y, floor.width, floor.height, floor.rotation, true, 1, 0.5);
			floorParticle.setStyle(0, 0x000000, 0, 0xFFFFFF, 0);
			var floorGroup:Group = new Group();
			floorGroup.addParticle(floorParticle);
			APEngine.addGroup(floorGroup);
			
			for (var i:int = 1; i < 35; i++) {
				var circle:Sprite = this["circle" + i] as Sprite;
				var group:Group = new Group()
				//var size:Number = Math.random();
				//var particle:CircleParticle = new CircleParticle(circle.x + Math.random()*10, circle.y + Math.random()*10, size * 3 + 3 , false, (size * 3 + 3) / 6 * 20);
				var particle:CircleParticle = new CircleParticle(circle.x, circle.y, circle.width / 2, false, 1.3);
				particle.setStyle(0, 0x000000, 0, 0xFFFFFF, 1);
				fluidParticles.push(particle);
				group.addParticle(particle);
				removeChild(circle);
				APEngine.addGroup(group);
				fluids.push(group);
			}
			
			var bottomCollidables:Array = new Array(floorGroup);
			
			for each(var fluidGroup1:Group in fluids) {
				bottomCollidables.push(fluidGroup1);
				var collidables:Array = new Array(bottle, floorGroup);
				
				for each(var fluidGroup2:Group in fluids) {
					if (fluidGroup1 != fluidGroup2) {
						collidables.push(fluidGroup2);
					}
				}
				
				fluidGroup1.addCollidableList(collidables);
			}
			
			addChild(floor);
			
			addEventListener(Event.ENTER_FRAME, enterFrameHandler);
		}
		
		private function floorMouseDownHandler(event:MouseEvent):void {
			offsetY = mouseY - floor.y;
			stage.addEventListener(MouseEvent.MOUSE_MOVE, stageMouseMoveHandler);
			stage.addEventListener(MouseEvent.MOUSE_UP, stageMouseUpHandler);
		}
		
		private function stageMouseMoveHandler(event:MouseEvent):void {
			floorTargetY = Math.max(mouseY - offsetY, min);
			floorTargetY = Math.min(floorTargetY, max);
		}
		
		private function stageMouseUpHandler(event:MouseEvent):void {
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, stageMouseMoveHandler);
			stage.removeEventListener(MouseEvent.MOUSE_UP, stageMouseUpHandler);
		}
		
		private function enterFrameHandler(event:Event):void {
			floor.y += (floorTargetY - floor.y) * 0.4;
			
			floorParticle.py = floor.y;
			
			APEngine.step();
			APEngine.paint();
			
			var lowest:Number = 0;
			for each(var particle:CircleParticle in fluidParticles) {
				if (particle.py > lowest) {
					lowest = particle.py;
				}
			}
			
			physicsContainer.liquid.y = Math.round(lowest);
			physicsContainer.liquid.height = 450 - physicsContainer.liquid.y;
			
			bitmapDataCopy.fillRect(bitmapDataCopy.rect, 0xFF000000);
			
			bitmapDataCopy.draw(physicsContainer, null, null, null, new Rectangle(0, 0, 250, 450), false);
			bitmapDataCopy.applyFilter(bitmapDataCopy, bitmapData.rect, new Point(0, 0), blurFilter);
			
			bitmapData.fillRect(bitmapData.rect, 0x00000000);
			bitmapData.threshold(bitmapDataCopy, bitmapDataCopy.rect, point, ">", 0XFF2b2b2b, 0x55FFFFFF, 0xFFFFFFFF, false);
			bitmapData.threshold(bitmapDataCopy, bitmapDataCopy.rect, point, ">", 0XFF2c2c2c, 0xBBFFFFFF, 0xFFFFFFFF, false);
			bitmapData.threshold(bitmapDataCopy, bitmapDataCopy.rect, point, ">", 0XFF2d2d2d, 0xFFFFFFFF, 0xFFFFFFFF, false);
			
		}
		
	}
	
}